@@ -14,7 +14,7 @@ use std::io;
 use std::iter;
 use std::net::IpAddr;
 use std::ops::Add;
-use std::process::{Command, Stdio};
+use std::process::{Command, ExitStatus, Stdio};
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::mpsc::Sender;
 use std::sync::{mpsc, Arc};
@@ -50,7 +50,7 @@ build_env: {},{}"#,
 );
 
 #[derive(Parser, Debug)]
-#[command(author, name = "gping", about = "Ping, but with a graph.", version=VERSION_INFO)]
+#[command(author, name = "gping", about = "Ping, but with a graph.", version = VERSION_INFO)]
 struct Args {
     #[arg(
         long,
@@ -80,6 +80,9 @@ struct Args {
     /// Resolve ping targets to IPv6 address
     #[arg(short = '6', conflicts_with = "ipv4")]
     ipv6: bool,
+    /// Interface to use when pinging.
+    #[arg(short = 'i', long)]
+    interface: Option<String>,
     #[arg(short = 's', long, help = "Uses dot characters instead of braille")]
     simple_graphics: bool,
     #[arg(
@@ -202,6 +205,7 @@ enum Update {
     Result(Duration),
     Timeout,
     Unknown,
+    Terminated(ExitStatus, String),
 }
 
 impl From<PingResult> for Update {
@@ -210,6 +214,7 @@ impl From<PingResult> for Update {
             PingResult::Pong(duration, _) => Update::Result(duration),
             PingResult::Timeout(_) => Update::Timeout,
             PingResult::Unknown(_) => Update::Unknown,
+            PingResult::PingExited(e, stderr) => Update::Terminated(e, stderr),
         }
     }
 }
@@ -268,13 +273,22 @@ fn start_ping_thread(
     watch_interval: Option<f32>,
     ping_tx: Sender<Event>,
     kill_event: Arc<AtomicBool>,
+    interface: Option<String>,
 ) -> Result<JoinHandle<Result<()>>> {
     let interval = Duration::from_millis((watch_interval.unwrap_or(0.2) * 1000.0) as u64);
     // Pump ping messages into the queue
-    let stream = ping_with_interval(host, interval)?;
+    let stream = ping_with_interval(host, interval, interface)?;
     Ok(thread::spawn(move || -> Result<()> {
         while !kill_event.load(Ordering::Acquire) {
-            ping_tx.send(Event::Update(host_id, stream.recv()?.into()))?;
+            match stream.recv() {
+                Ok(v) => {
+                    ping_tx.send(Event::Update(host_id, v.into()))?;
+                }
+                Err(_) => {
+                    // Stream closed, just break
+                    return Ok(());
+                }
+            }
         }
         Ok(())
     }))
@@ -364,6 +378,7 @@ fn main() -> Result<()> {
                 args.watch_interval,
                 key_tx.clone(),
                 std::sync::Arc::clone(&killed),
+                args.interface.clone(),
             )?);
         }
     }
@@ -399,6 +414,16 @@ fn main() -> Result<()> {
                     Update::Result(duration) => app.update(host_id, Some(duration)),
                     Update::Timeout => app.update(host_id, None),
                     Update::Unknown => (),
+                    Update::Terminated(e, _) if e.success() => {
+                        break;
+                    }
+                    Update::Terminated(e, stderr) => {
+                        eprintln!(
+                            "There was an error running ping: {}\nStderr: {}\n",
+                            e, stderr
+                        );
+                        break;
+                    }
                 };
                 terminal.draw(|f| {
                     // Split our
@@ -425,13 +450,14 @@ fn main() -> Result<()> {
                             .direction(Direction::Horizontal)
                             .constraints(
                                 [
-                                    Constraint::Percentage(28),
-                                    Constraint::Percentage(12),
-                                    Constraint::Percentage(12),
-                                    Constraint::Percentage(12),
-                                    Constraint::Percentage(12),
-                                    Constraint::Percentage(12),
-                                    Constraint::Percentage(12),
+                                    Constraint::Percentage(30),
+                                    Constraint::Percentage(10),
+                                    Constraint::Percentage(10),
+                                    Constraint::Percentage(10),
+                                    Constraint::Percentage(10),
+                                    Constraint::Percentage(10),
+                                    Constraint::Percentage(10),
+                                    Constraint::Percentage(10),
                                 ]
                                 .as_ref(),
                             )
@@ -480,15 +506,17 @@ fn main() -> Result<()> {
             },
         }
     }
+    killed.store(true, Ordering::Relaxed);
 
+    disable_raw_mode()?;
+    execute!(terminal.backend_mut())?;
+    terminal.show_cursor()?;
+
+    let new_size = terminal.size()?;
+    terminal.set_cursor(new_size.width, new_size.height)?;
     for thread in threads {
         thread.join().unwrap()?;
     }
 
-    disable_raw_mode()?;
-    execute!(terminal.backend_mut(),)?;
-    terminal.show_cursor()?;
-    let new_size = terminal.size()?;
-    terminal.set_cursor(new_size.width, new_size.height)?;
     Ok(())
 }
